package BusinessSimulation "Dynamic modeling for business, economics, and ecology"
  annotation(uses(Modelica(version = "4.0.0")), version = "2.2.0", versionDate = "2024-03-04", revisionId = "", Documentation(info = "<html>
<p class=\"aside\">This information is part of the Business Simulation&nbsp;Library (BSL). Please support this work and <a href=\"https://www.paypal.com/donate/?hosted_button_id=GXVZT8LD7CFXN\" style=\"font-weight:bold; color:orange; text-decoration:none;\">▶&nbsp;donate</a>.</p>
<p>The <span style=\"color: #000080;\"><strong>BUSINESS SIMULATION LIBRARY (BSL)</strong></span> supports modeling and simulation of dynamical systems in the social sciences and ecology. Following John Sterman&nbsp;[<a href=\"modelica://BusinessSimulation.UsersGuide.References\">3</a>], the term \"Business Simulation\" is used in a broad sense, as modeling in these domains is concerned with control, decision making, and management of resources.</p>
<p>The <span style=\"color: #000080;\"><strong>BSL</strong> </span>adopts the widely recognized <span style=\"color: #000080;\"><strong>System Dynamics</strong></span> approach and modeling paradigm introduced by Jay W. Forrester [<a href=\"modelica://BusinessSimulation.UsersGuide.References\">1</a>], [<a href=\"modelica://BusinessSimulation.UsersGuide.References\">2</a>].&nbsp;System dynamics provides a versatile, foundational modeling framework, ideally suited for modeling, simulating, and analyzing strategic business or public policy issues.</p>
<p>Unlike existing approaches, the <span style=\"color: #000080;\"><strong>BSL</strong></span> utilizes Modelica's acausal connectors to accurately model the flow of <em>conserved quantities</em> such as matter, energy, or information, as well as to represent instantaneous <em>information signal flows</em> through causal connections. Using pre-built components and hierarchical modeling, even complex models can be developed in a fast and reliable manner with the added benefits of embedded documentation and compact presentation. A comprehensive introduction to the BSL and cyber-physical modeling is available in a recent paper [<a href=\"modelica://BusinessSimulation.UsersGuide.References\">28</a>] by the principal author.</p>
<p>To <strong style=\"color: #000080;\">get started with the library</strong>, consider exploring the following resources:</p>
<p><span style=\"color:orange;\">&#9658;</span><strong><a href=\"modelica://BusinessSimulation.UsersGuide\">&nbsp;User's Guide</a></strong></p>
<p><span style=\"color:orange;\">&#9658;</span><strong><a href=\"modelica://BusinessSimulation.Examples\">&nbsp;Examples</a></strong></p>
<p>For information regarding <strong style=\"color: #000080;\">further support options</strong>, <strong style=\"color: #000080;\">error reporting</strong>, and <strong style=\"color: #000080;\">feature suggestions</strong> click <a href=\"modelica://BusinessSimulation.UsersGuide.Contact\">here</a><br><br>
The main repository for this library can be found at <a href=\"https://github.com/bslMS/BusinessSimulation\">https://github.com/bslMS/BusinessSimulation</a>.<br>
A dedicated Wolfram System Modeler version can be obtained from <a href=\"https://www.wolfram.com/system-modeler/libraries/business-simulation/\">System Modeler Modelica Library Store.</a></p>
<br>
<hr>
<h4 style=\"color: #000080;\">Principal Author and Support</h4>
<p>
<a href=\"modelica://BusinessSimulation.UsersGuide.Contact\">
<img src=\"modelica://BusinessSimulation/Resources/Images/UsersGuide/Contact/ContactInformation.png\" alt=\"ContactInformation\" width=\"400\" title=\"Click here for contact and support information\">
</a></p>
<hr>
<h4 style=\"color: #000080;\">Your Contribution is Needed</h4> 
<p>Join me in shaping the future of business strategy and analysis. The Business Simulation Library (BSL), available under the EUPL-1.2, offers cutting-edge tools for modeling and simulation, empowering professionals and academics alike. Your donation directly contributes to enhancing BSL’s capabilities, ensuring it remains a state-of-the-art resource available to all, free of charge.</p> <p>Your support enables continuous improvement, from refining existing features to developing new ones, all while keeping BSL accessible. If BSL has been a valuable asset in your work or studies, please consider making a contribution via PayPal. Every donation, no matter the size, plays a crucial role in our journey towards advancing business simulation technologies.</p> <p>Thank you for considering a donation.</p>
<p>
<a href=\"https://www.paypal.com/donate/?hosted_button_id=GXVZT8LD7CFXN\"><img height=\"28\" style=\"border:0px;height:28px;\" src=\"modelica://BusinessSimulation/Resources/Images/donate_button.svg\" border=\"0\" alt=\"Donate with PayPal\" title=\"Donate with PayPal\" /></a>
</p>
<br>
<hr>
<h4 style=\"color: #000080;\">Acknowledgements</h4>
<p>I am greatly indebted to <a href=\"https://www.wolfram.com/\">Wolfram Research, Inc.</a> and <a href=\"https://www.wolframmathcore.com/\">Wolfram MathCore AB</a> for their invaluable support and contributions to the Business Simulation Library. Their expertise and resources have played a crucial role in the development and success of this project. I extend my sincerest gratitude for their generosity and collaboration.</p>
<br>
<hr>
<h4 style=\"color: #000080;\">Releases</h4>
<ul>
<li><span style=\"color: #000080;\"><a href=\"modelica://BusinessSimulation.UsersGuide.ReleaseNotes.Version_2_2_0\">Version 2.2.0 &nbsp;(2024-03-04)</a></span></li>
<li><span style=\"color: #000080;\"><a href=\"modelica://BusinessSimulation.UsersGuide.ReleaseNotes.Version_2_1_1\">Version 2.1.1 &nbsp;(2023-05-02)</a></span></li>
<li><span style=\"color: #000080;\"><a href=\"modelica://BusinessSimulation.UsersGuide.ReleaseNotes.Version_2_1_0\">Version 2.1.0 &nbsp;(2022-12-03)</a></span></li>
<li><span style=\"color: #000080;\"><a href=\"modelica://BusinessSimulation.UsersGuide.ReleaseNotes.Version_2_0_0\">Version 2.0.0 &nbsp;(2021-12-06)</a></span></li>
<li><span style=\"color: #000080;\"><a href=\"modelica://BusinessSimulation.UsersGuide.ReleaseNotes.Version_1_0_1\">Version 1.0.1 &nbsp;(2021-02-02)</a></span></li>
<li><span style=\"color: #000080;\"><a href=\"modelica://BusinessSimulation.UsersGuide.ReleaseNotes.Version_1_0_0\">Version 1.0.0 &nbsp;(2020-11-29)</a></span></li>
</ul>
<p>Starting with release <code>v2.2.0</code>, a dedicated Wolfram System Modeler version is published with the build identifier <code>-wsm</code>.</p>
<hr>
<p><span style=\"color: #000080;\"><strong>Copyright &copy; 2020 Guido Wolf Reichert</strong></span><br>Licensed unter the European Union Public Licence (EUPL), Version 1.2 or later (the \"License\")</p>
<p>You may not use this work except in compliance with the License. You may obtain a copy of the License at:<br> <a href=\"https://eur-lex.europa.eu/eli/dec_impl/2017/863/oj\">https://eur-lex.europa.eu/eli/dec_impl/2017/863/oj</a>&nbsp;(the English text for&nbsp;<a href=\"modelica://BusinessSimulation.UsersGuide.Licence\">EUPL-1.2</a>&nbsp;is included in the <a href=\"modelica://BusinessSimulation.UsersGuide\">UsersGuide</a>)</p>
<p>Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an \"AS IS\" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either expressed or implied. See the License for the specific language governing permissions and limitations under the License.</p>
<p><strong>Modelica&copy; </strong>is a registered trademark of the Modelica Association.</p>
<hr>
<br>
<p>
The freely available <strong>Lato fonts family</strong> is used by library classes; for the best visual experience it is highly recommended to install this font on your computer, which is available for download from its <a href=\"https://www.latofonts.com/lato-free-fonts/\">official site</a> or&mdash;in an earlier version&mdash;from the <a href=\"https://fonts.google.com/specimen/Lato?query=Lato\">Google Fonts</a> website.
</p>
</html>"), __Wolfram(itemFlippingEnabled = true, Units(symbols = {BaseUnit(symbol = "bit", name = "bit", unitSystems = {"universal"}, allowPrefix = true, WL = "\"Bits\"", description = "The fundamental unit of digital information, representing a binary state of either 0 or 1."), BaseUnit(symbol = "each", name = "each", unitSystems = {"universal"}, presentation = "#", WL = "\"DimensionlessUnit\"", description = "Basic counting unit for the number of elementary units or entities."), BaseUnit(symbol = "FTE", name = "full-time equivalent", unitSystems = {"universal"}, description = "Basic measurement unit for labor and workload in need of further specification in models.", WL = "IndependentUnit[\"FTE\"]"), BaseUnit(symbol = "XXX", name = "unspecified currency", unitSystems = {"universal"}, presentation = "¤", WL = "IndependentUnit[\"¤\"]", description = "No (real) currency according to ISO 4217, which is taken to mean an arbitrary, unspecified currency here."), BaseUnit(symbol = "EUR", name = "euro", unitSystems = {"universal"}, presentation = "€", WL = "\"Euros\"", description = "The euro according to ISO 4217."), BaseUnit(symbol = "USD", name = "united States dollar", unitSystems = {"universal"}, presentation = "$", WL = "\"USDollars\"", description = "The United States dollar according to ISO 4217."), BaseUnit(symbol = "GBP", name = "pound sterling", unitSystems = {"universal"}, presentation = "£", WL = "\"BritishPounds\"", description = "The British pound sterling according to ISO 4217."), BaseUnit(symbol = "JPY", name = "japanese yen", unitSystems = {"universal"}, presentation = "JP¥", WL = "\"Yen\"", description = "The Japanese yen according to ISO 4217."), BaseUnit(symbol = "CNY", name = "renminbi", unitSystems = {"universal"}, presentation = "CN¥", WL = "\"ChineseYuan\"", description = "The Chinese yuan according to ISO 4217."), DerivedUnit(symbol = "byte", name = "byte", source = "bit", scale = 8., presentation = "B", allowPrefix = true, description = "An octet with metric multiples in accordance with SI.", WL = "\"Bytes\""), DerivedUnit(symbol = "wk", name = "week", source = "s", scale = 604800., WL = "\"Weeks\""), DerivedUnit(symbol = "mo", name = "gregorian month", source = "s", scale = 2629746., description = "A 12th of a mean Gregorian calendar year.", WL = "\"GregorianMonths\""), DerivedUnit(symbol = "qtr", name = "gregorian quarter year", source = "s", scale = 7889238., description = "A quarter of a the mean Gregorian calendar year."), DerivedUnit(symbol = "yrCal", name = "gregorian year", source = "s", scale = 31556952., presentation = "y", description = "A mean Gregorian calendar year will average across common and leap year definitions (365.2425 days).", WL = "\"GregorianYears\""), DerivedUnit(symbol = "a", name = "julian year", source = "d", scale = 365.25, description = "The Julian year is a simple average over three common years and one leap year. It is typically used in astronomy.", WL = "\"JulianYears\""), DerivedUnit(symbol = "bp", name = "basis point", source = "1", scale = 0.0001, presentation = "‱", WL = "\"BasisPoints\""), DerivedUnit(symbol = "pair", name = "pair", source = "each", scale = 2., description = "Counting unit."), DerivedUnit(symbol = "doz", name = "dozen", source = "each", scale = 12., description = "Counting unit.", WL = "\"Dozen\""), DerivedUnit(symbol = "score", name = "score", source = "each", scale = 20., description = "Counting unit.", WL = "\"Scores\""), DerivedUnit(symbol = "gross", name = "gross", source = "each", scale = 144., description = "Counting unit.", WL = "\"Gross\""), DerivedUnit(symbol = "ream", name = "ream", source = "each", scale = 500., description = "Counting unit—here taken to simply mean \"500 each\" instead of \"500 sheets.\""), DerivedUnit(symbol = "thousand", name = "thousand", source = "each", scale = 1000., presentation = "k", description = "Counting unit.", WL = "\"Thousand\""), DerivedUnit(symbol = "million", name = "million", source = "each", scale = 1.e6, presentation = "M", description = "Counting unit.", WL = "\"Million\""), DerivedUnit(symbol = "billion", name = "billion", source = "each", scale = 1.e9, presentation = "B", description = "Counting unit (short scale definition).", WL = "\"Billion\""), DerivedUnit(symbol = "trillion", name = "trillion", source = "each", scale = 1.e12, presentation = "T", description = "Counting unit (short scale definition).", WL = "\"Trillion\""), DerivedUnit(symbol = "mpd", name = "miles per day", source = "mi/d", WL = "\"Miles\" / \"Days\""), DerivedUnit(symbol = "t", name = "metric ton", source = "kg", scale = 1000., allowPrefix = true, WL = "\"MetricTons\"", ignoreShadowing = true)}, quantities = {Quantity(names = {"DataCapacity"}, displayUnits = {"bit", "Mbit", "kbyte", "Mbyte", "Gbyte", "Tbyte", "Pbyte", "Zbyte"}), Quantity(names = {"Money"}, displayUnits = {"XXX", "EUR", "USD", "GBP", "JPY", "CNY"}), Quantity(names = {"Labor"}, displayUnits = {"FTE"}), Quantity(names = {"Time"}, displayUnits = {"wk", "mo", "qtr", "yrCal"}), Quantity(names = {"Mass"}, displayUnits = {"t", "kt", "Mt"}), Quantity(names = {"Rate"}, displayUnits = {"1/wk", "1/mo", "percent/h", "1/yrCal", "percent/d", "percent/wk", "percent/mo", "percent/yrCal", "bp/h", "bp/d", "bp/wk", "bp/mo", "bp/yrCal"}), Quantity(names = {"Throughput"}, displayUnits = {"each/h", "each/d", "each/wk", "each/mo", "each/yrCal", "thousand/h", "thousand/d", "thousand/wk", "thousand/mo", "thousand/yrCal", "million/h", "million/d", "million/wk", "million/mo", "million/yrCal", "billion/yrCal", "trillion/yrCal"}), Quantity(names = {"MassFlowRate"}, displayUnits = {"kg/h", "kg/d", "kg/wk", "kg/mo", "kg/yrCal", "t/d", "t/yrCal"}), Quantity(names = {"Amount", "Population", "People", "TangibleAssets", "IntangibleAssets", "Material", "Orders"}, displayUnits = {"thousand", "million", "billion", "trillion"}), Quantity(names = {"LaborGrowthRate"}, displayUnits = {"FTE/mo", "FTE/yrCal"}), Quantity(names = {"VolumeFlowRate"}, displayUnits = {"m3/wk", "m3/mo", "m3/yrCal", "L/h", "L/d", "L/wk", "L/mo", "L/yrCal", "gal/d", "gal/yrCal"}), Quantity(names = {"Velocity"}, displayUnits = {"km/d", "mpd", "km/yrCal"}), Quantity(names = {"MonetaryFlow"}, displayUnits = {"XXX/h", "XXX/d", "XXX/mo", "XXX/yrCal", "EUR/h", "EUR/d", "EUR/mo", "EUR/yrCal", "USD/h", "USD/d", "USD/mo", "USD/yrCal", "GBP/h", "GBP/d", "GBP/mo", "GBP/yrCal", "JPY/h", "JPY/d", "JPY/mo", "JPY/yrCal", "CNY/h", "CNY/d", "CNY/mo", "CNY/yrCal"}), Quantity(names = {"DataTransferRate"}, displayUnits = {"bit/s", "kbit/s", "Gbit/s", "Mbit/s", "Tbit/s", "Mbyte/s", "Gbyte/s", "Tbyte/s", "Tbyte/d", "Pbyte/mo", "Zbyte/yrCal"}), Quantity(names = {"Power"}, displayUnits = {"GW", "TW", "PW", "MW.h/yrCal", "GW.h/yrCal", "TW.h/yrCal", "kW.h/d", "MW.h/d"}), Quantity(names = {"Energy"}, displayUnits = {"TW.h", "GW.h", "PW.h"})})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {76, 112, 136}, fillColor = {113, 166, 201}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-100, -100}, {100, 100}}, radius = 25), Rectangle(visible = true, origin = {0.142, -2.447}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, lineThickness = 8, extent = {{-62.191, -62.447}, {62.191, 62.447}}), Text(visible = true, origin = {0, -30}, textColor = {255, 255, 255}, extent = {{-60, -30}, {60, 30}}, textString = "BSL", fontName = "Lato", textStyle = {TextStyle.Bold}), Polygon(visible = true, origin = {25.123, 60}, rotation = -990, lineColor = {76, 112, 136}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, points = {{-10, 10}, {10, 0}, {-10, -10}, {-10, 10}}), Polygon(visible = true, origin = {-23.947, 60}, rotation = -1170, lineColor = {255, 255, 255}, fillColor = {76, 112, 136}, fillPattern = FillPattern.Solid, points = {{-10, 10}, {10, 0}, {-10, -10}, {-10, 10}}), Rectangle(visible = true, origin = {-63.017, -0.581}, lineColor = {76, 112, 136}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 3, extent = {{-8.5, -9.419}, {8.5, 9.419}}), Rectangle(visible = true, origin = {61.5, 0}, lineColor = {255, 255, 255}, fillColor = {76, 112, 136}, fillPattern = FillPattern.Solid, lineThickness = 3, extent = {{-8.5, -9.419}, {8.5, 9.419}})}), Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5}), graphics = {Text(visible = true, origin = {0, 15}, textColor = {128, 128, 128}, extent = {{-26.627, -22.475}, {26.627, 22.475}}, textString = "ⓘ", fontName = "Arial"), Text(visible = true, origin = {0, -15}, textColor = {128, 128, 128}, extent = {{-140, -6}, {140, 6}}, textString = "Documentation Only", fontName = "Arial"), Text(visible = true, origin = {-0, -31}, textColor = {128, 128, 128}, extent = {{-140, -6}, {140, 6}}, textString = "Press F1", fontName = "Arial", textStyle = {TextStyle.Bold})}));
end BusinessSimulation;
